;**************************** Pulse regulator ***************************
;
;   File name   :  main.asm
;   AUTHOR      :  Valentin Kulikov, VKU2009
;   E-mail		:  valentin@vvku.eu
;   VERSION     :  0.11
;   DATE        :  29.MAR.09
;   Internal osc:  9,6 MHz
;   MCU			:  ATtiny 13
;   Compiler    :  Atmel AVR Assembler
;   Contents    :  
;
;   Function	:  Pulse regulator for truck air conditioning DC motor
;				   ON = 25%, A=50%, B=75%, C=100%
;				   Integrated rise and fall power increase
;
;   Additional	:  Note: EEPROM not used
;   FUSES		:  see fuses.jpg
;	Note		:  
;************************************************************************

; Define files 
.include	"tn13def.inc"

.def	Temp	=r16	;temporary register
.def	Temp1	=r17	;temporary register #1
.def	dly1	=r19	;delay register #1
.def	dly2	=r20	;delay register #2

;************************* IN/OUT pin definition ************************

.equ	A	= 5		;Input 50%
.equ	B	= 3		;Input 75%
.equ	C	= 4		;Input 100%

.equ	LED	= 1		;LED indication
.equ	MOT	= 0		;MOTOR Output 

;************************** Interrupt Vector Table **********************
; 1 Byte each:

.ORG $0000

		rjmp 	RST 		; Reset handler
		reti				; IRQ0 handler
		rjmp	MAIN		; Pin change handler
		reti				; Timer0 overflow handler
		reti				; EEPROM Ready handler
		reti				; Analog Comparator handler
 
		.db	"Pulse Regulator    ",0
		.db	"Ver. 1.0 by VKU2009",0 


;***MAIN*******************************************

RST:	
		clr		temp
		out		PORTB, temp			;reset PORTB
		out		DDRB, temp			;set all pins as inputs

		cbi 	PORTB, A			;disable Pull-up resistor 35-115kOhm
		cbi 	PORTB, B			;disable Pull-up resistor 35-115kOhm
		cbi 	PORTB, C			;disable Pull-up resistor 35-115kOhm (just for sure)

		sbi		DDRB, LED			;set pin as output
		cbi		PORTB, LED			;set to low

		sbi		DDRB, MOT			;set pin as output
		cbi		PORTB, MOT			;set to low

		ldi		temp, 0b00011000	;Set Watch Dog change enable and WDE
		out		WDTCR, temp
		ldi		temp, 0b00100000	;Set Watchdog prescaler ~4 sec
		out		WDTCR, temp
		

									; Initial PWM
        ldi   	R16,0b10000011		;PB0 activated
 		out   	TCCR0A,R16       	;Fast PWM Mode, Clear on OC0A
		ldi  	R16,0b00000100
		out  	TCCR0B,R16       	;Used fclk/256 prescaller

		ldi		temp, 0				;Ratio 0%	
RISE25:								;Rise to 25% output ratio
		out   	OCR0A, temp
		rcall	dly5ms
		cpi		temp, 64
		inc		temp
		brlo	RISE25
		sbi		PORTB, LED			;set to high      

MAIN:
		wdr
		ldi		temp1, 64			;check ABC input status
		sbic	PINB, A
		ldi		temp1, 128
		sbic	PINB, B
		ldi		temp1, 192
		sbic	PINB, C
		ldi		temp1, 255

		in		temp, OCR0A			;get actual ratio settings
		cp		temp1, temp			;if actual lower than set go to RISE
		brsh	RISE	

FALL:								;Fall to set ratio value
		out   	OCR0A,temp
		dec		temp
		rcall	dly5ms
		cp		temp1, temp
		brlo	FALL

		rjmp	MAIN

RISE:								;Rise up to set ratio value
		out   	OCR0A,temp
		inc		temp
		rcall	dly5ms
		cp		temp, temp1
		brlo	RISE
		
		rjmp	MAIN	
									;Delay Function
dly5ms:
		wdr
delay1: ldi   	dly1,65 
delay2: dec   	dly1
		brne  	delay2           
		dec   	dly2
		brne  	delay1 
		ret                   
.exit


;That all folks
;VKU2009
